% script file spanalyse.m
% analyse the behavior of sparse matrices
% 02.05.05 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc; clear all; close all;

ndom = [20:1:70];

sparse_inv_time = zeros(size(ndom));
full_inv_time = zeros(size(ndom));

sparse_mult_time = zeros(size(ndom));
full_mult_time = zeros(size(ndom));

sparse_storage = zeros(size(ndom));
full_storage = zeros(size(ndom));

for ctr = 1:length(ndom);
  n = ndom(ctr);  
  disp(n);

  A = SparseMatrixExample(n);
  B = full(A);

  % compare the storage
  a = whos('A');
  sparse_storage(ctr) = a.bytes;
  
  b = whos('B');
  full_storage(ctr) = b.bytes;
  
  
    
  f = rand(n^2,1);
  
  % time for inverting a sparse matrix
  tic;
  x=A\f;
  sparse_inv_time(ctr) = toc;
    
  % time for inverting a full matrix
  tic;
  x = B\f;
  full_inv_time(ctr) = toc;
  
  x = rand(n^2,1);
  
  tic;
  y = MMult_full(B,x,n^2);
  full_mult_time(ctr) = toc;
  
  [I,J,S] = find(A);
  tic;
  z = MMult_sparse(I,J,S,x,n^2);  
  sparse_mult_time(ctr) = toc;
  
end
  

dim = ndom.^2;
figure(1)
subplot(1,3,1)
loglog(dim,sparse_inv_time,'b*',dim,full_inv_time,'ro',dim,dim,'k',dim,dim.^2,'m',dim,dim.^3,'c');
xlabel('dim'); ylabel('time')
legend('sparse','full','dim','pow(dim,2)','pow(dim,3)');
title('solving a system');
%axis equal; 
grid on

subplot(1,3,2)
loglog(dim,sparse_mult_time,'b*',dim,full_mult_time,'ro',dim,dim,'k',dim,dim.^2,'m',dim,dim.^3,'c');
xlabel('dim'); ylabel('time')
legend('sparse','full','dim','pow(dim,2)','pow(dim,3)');
title('multiplying with a vector');
%axis equal; 
grid on

subplot(1,3,3)
loglog(dim,sparse_storage,'b*',dim,full_storage,'ro',dim,dim,'k',dim,dim.^2,'m',dim,dim.^3,'c');
xlabel('dim'); ylabel('time')
legend('sparse','full','dim','pow(dim,2)','pow(dim,3)');
title('storage');
%axis equal; 
grid on





